import sys
import asyncio
import argparse
import json

from fastapi import FastAPI, Request
from fastapi.responses import StreamingResponse
from fastapi import status
from hypercorn.config import Config
from hypercorn.asyncio import serve
import ipaddress

# Setup Parser
parser = argparse.ArgumentParser()
parser.add_argument('-c', '--config', type=argparse.FileType('r'),
                    default='./config', help='Path to config file in JSON format.')
# parser.add_argument('-db', '--database', type=argparse.FileType('r'),  # TODO: read+write
#                     default='./db.json', help='Path to database file in JSON format.')

args = parser.parse_args(sys.argv[1:])

# Load Config
config = json.load(args.config)


api = FastAPI()

BUFFER_SIZE = 1024 * 4  # 4KB
HOST = '127.0.0.1'
PORT = 9250


async def generate_test_data(size: int | str, max_size=1024 * 1024) -> bytes:
    size_left = None

    try:
        size_left = int(size)
    except ValueError:  # treat as string
        units = {
            'GB': 10 ** 9, 'GiB': 2 ** 30,
            'MB': 10 ** 6, 'MiB': 2 ** 20,
            'KB': 10 ** 3, 'KiB': 2 ** 10,
            'B': 1
        }

        for unit in units:
            if size.endswith(unit):
                size_left = int(float(size.removesuffix(unit)) * units[unit])
                break

    print('size_left', size_left)

    # yield data
    while size_left > BUFFER_SIZE:
        size_left -= BUFFER_SIZE
        yield b'\0' * BUFFER_SIZE
    else:
        yield b'\0' * size_left


def check_policies(ip: str) -> None:
    network = ipaddress.ip_network(ip)
    print(network)

@api.get('/')
async def test_data(size: str, request: Request) -> StreamingResponse:
    try:
        check_policies(request.client.host)

        return StreamingResponse(
            status_code=status.HTTP_200_OK,
            content=generate_test_data(size)
        )
    # except TooManyRequestsError as err:
    #     ...
    # except BlockedError as err:
    #     ...
    # except OutOfQuotaError as err:
    #     ...
    except err:
        raise err
        pass


def main():
    asyncio.run(serve(
        api,
        Config().from_object({'bind': [f'{HOST}:{PORT}']})
    ))


if __name__ == '__main__':
    main()
